package ru.littlevictor.radio;

import android.content.Context;
import android.preference.PreferenceManager;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import java.util.SortedMap;

public class CodePage {

    private Context context;
    private String[] charsetNamesArray;

    public CodePage(Context context) {
        this.context = context;
        ArrayList<String> charsetNamesArrayList = new ArrayList<String>();
        SortedMap charSets = Charset.availableCharsets();
        Set charsetNames = charSets.keySet();
        for (Iterator e = charsetNames.iterator(); e.hasNext();) {
            charsetNamesArrayList.add(e.next().toString());
        }
        charsetNamesArray = charsetNamesArrayList.toArray(new String[charsetNamesArrayList.size()]);
    };

    // -----------------------------------------------------------------------------
    public String[] getNames() {
        return charsetNamesArray;
    };

    // -----------------------------------------------------------------------------
    public int getIndex() {
        int index;
        try {
            index = Integer.parseInt(PreferenceManager.getDefaultSharedPreferences(context).getString("code_page", ""));
        } catch(NumberFormatException nfe) {
            index = 0;
            for(int i = 0; i < charsetNamesArray.length; i++) {
                if(charsetNamesArray[i].equalsIgnoreCase("UTF-8")) {
                    index = i;
                    setIndex(index);
                    break;
                }
            }
        }
        if(index < 0 || index > charsetNamesArray.length - 1) {
            for(int i = 0; i < charsetNamesArray.length; i++) {
                if(charsetNamesArray[i].equalsIgnoreCase("UTF-8")) {
                    index = i;
                    setIndex(index);
                    break;
                }
            }
        }
        return index;
    };

    // -----------------------------------------------------------------------------
    public void setDefault() {
        for(int i = 0; i < charsetNamesArray.length; i++) {
            if(charsetNamesArray[i].equalsIgnoreCase("UTF-8")) {
                setIndex(i);
                break;
            }
        }
    };

    // -----------------------------------------------------------------------------
    public void setIndex(int index) {
        PreferenceManager.getDefaultSharedPreferences(context).edit().putString("code_page", Integer.toString(index)).apply();
    };

    // -----------------------------------------------------------------------------
    public String getName() {
        return charsetNamesArray[getIndex()];
    };
};
