package ru.littlevictor.radio;

import android.annotation.SuppressLint;
import android.content.Context;
import android.net.Uri;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;
import java.util.ArrayList;

//import java.io.FileNotFoundException;
//import java.io.FileReader;
//import java.io.FileWriter;
//import java.nio.charset.Charset;
//import java.util.SortedMap;
//import android.content.SharedPreferences;
//import android.preference.PreferenceManager;

public class Data {

	//-----------------------------------------------------------------------------

	public ArrayList<String> names;
    public ArrayList<String> urls;
    public ArrayList<String> index;
	public String errorMessage = "";
	private String path = "";
	private String parentPath = "";
	private Context context;
//    private SharedPreferences sharedPreferences;
    private final CodePage codePage;

	//-----------------------------------------------------------------------------
	public Data(Context context, String filePath) {
		this.context = context;
        this.codePage = new CodePage(context);
//        this.sharedPreferences = PreferenceManager.getDefaultSharedPreferences(context);
        this.names = new ArrayList<String>();
        this.urls = new ArrayList<String>();
        this.index = new ArrayList<String>();
		if(!readFile(context, filePath)) {
			getDefaultList(context);
            this.path = Storage.getPath(context, Storage.MAIN_LIST);
			if(filePath.equalsIgnoreCase(path)) {
				writeList();
			}
		} else {
            this.path = filePath;
		}
	};
	
	//-----------------------------------------------------------------------------
	public String getPath() {
		return path;
	};
	
	//-----------------------------------------------------------------------------
	public String getParentPath() {
		return parentPath;
	};
	
	//-----------------------------------------------------------------------------
	public boolean writeList(String filePath) {
		if(writeFile(filePath)) return true;
		return false;
	};
	
	//-----------------------------------------------------------------------------
	public boolean writeList() {
		if(path == null) path = Storage.getPath(context, Storage.MAIN_LIST);
		if(writeFile(path)) return true;
		return false;
	};
	
	//-----------------------------------------------------------------------------
	public int size() {
		return names.size();
	};
	
	//-----------------------------------------------------------------------------
	private void getDefaultList (Context context) {
		names.clear();
		urls.clear();
        index.clear();
		String[] defaultNames = context.getResources().getStringArray(R.array.default_names);
		String[] defaultUrls = context.getResources().getStringArray(R.array.default_urls);
        for (int i = 0; i < defaultNames.length; i++) {
        	names.add(defaultNames[i]);
            urls.add(defaultUrls[i]);
            index.add(defaultNames[i] + defaultUrls[i] + i);
        };
	};
	
	//-----------------------------------------------------------------------------
	private boolean parseUri(String url) {
	    Uri uri;
		String scheme;
		String path;
	    try
	    {
		    uri = Uri.parse(url);
		    scheme = uri.getScheme();
			path = uri.getPath();
	    }
	    catch (Exception e) 
	    {
            return false;
	    }
		// -----------------------------------------------------------------------------
	    if (uri == null || scheme == null || path == null) 
	    {
            return false;
	    }
		return true;
	};
	
	//-----------------------------------------------------------------------------
	private boolean readFile(Context context, String filePath) {
		File file;
		try {
			file = new File(filePath);
		} catch (Exception e) {
			errorMessage = context.getResources().getString(R.string.invalid_file_path) + ": " + filePath;
			return false;
		}
		parentPath = file.getParent();
		if(file.canRead()) {
			names.clear();
			urls.clear();
            index.clear();
			int format = 0;
			String name = "";
			try {
                FileInputStream fileInputStream;
                InputStreamReader inputStreamReader;
                fileInputStream = new FileInputStream(filePath);
                inputStreamReader = new InputStreamReader(fileInputStream, codePage.getName());
                BufferedReader br = new BufferedReader(inputStreamReader);
                String str;
                while ((str = br.readLine()) != null) {
                    switch (format) {
                    case 0:
                        if (str.equalsIgnoreCase("")) continue;
                        if (str.startsWith("#EXTM3U")) {
                            format = 1;
                        } else if (str.startsWith("[playlist]")) {
                            format = 2;
                        } else {
                            if (parseUri(str)) {
                                format = 3;
                                names.add(str);
                                urls.add(str);
                            } else continue;
                        }
                        break;
                    case 1:
                        if (str.equalsIgnoreCase("")) continue;
                        if (str.startsWith("#EXTINF:")) {
                            name = str.substring(str.indexOf(",") + 1);
                        } else {
                            if (parseUri(str)) {
                                names.add(name);
                                urls.add(str);
                            } else {
                                names.add(name);
                                urls.add("file://" + parentPath + "/" + str);
                            }
                        }
                        break;
                    case 2:
                        if (str.equalsIgnoreCase("")) continue;
                        if (str.startsWith("File")) {
                            urls.add(str.substring(str.indexOf("=") + 1));
                        } else if (str.startsWith("Title")) {
                            names.add(str.substring(str.indexOf("=") + 1));
                        } else {
                            if (parseUri(str)) {
                                names.add(str);
                                urls.add(str);
                            } else continue;
                        }
                        break;
                    case 3:
                        if (str.equalsIgnoreCase("")) continue;
                        if (parseUri(str)) {
                            names.add(str);
                            urls.add(str);
                        } else continue;
                        break;
                    }
                }
                br.close();
                if (names.size() == urls.size()) {
                    if (names.size() == 0) {
                        names.add("");
                        urls.add("");
                        index.add("");
                        return true;
                    } else {
                        for (int i = 0; i < names.size(); i++) {
                            index.add("" + i);
                        };
                    }
                    errorMessage = "";
                    return true;
                } else {
                    names = new ArrayList<String>(urls);
                    for (int i = 0; i < names.size(); i++) {
                        index.add("" + i);
                    };
                    errorMessage = context.getResources().getString(R.string.invalid_list_format);
                    return true;
                }
            } catch (UnsupportedCharsetException e) {
                errorMessage = e.getMessage();
                return false;
			} catch (IllegalCharsetNameException e) {
				errorMessage = e.getMessage();
				return false;
			} catch (IOException e) {
				errorMessage = e.getMessage();
				return false;
			}
		} else {
			errorMessage = context.getResources().getString(R.string.invalid_file_path) + ": " + filePath;
			return false;
		}
	};
	
	//-----------------------------------------------------------------------------
	@SuppressLint("DefaultLocale")
	private boolean writeFile(String filePath) {
        FileOutputStream fileOutputStream;
        OutputStreamWriter outputStreamWriter;
		String file = filePath.toLowerCase();
		try {
			if(file.endsWith(".pls")) {
                fileOutputStream = new FileOutputStream(filePath);
                outputStreamWriter = new OutputStreamWriter(fileOutputStream, "UTF-8");
                BufferedWriter bw = new BufferedWriter(outputStreamWriter);
				String str = "[playlist]\nnumberofentries=" + String.valueOf(names.size()) + "\n";
				for(int i = 0; i < names.size(); i++) {
					str += "File" + String.valueOf(i + 1) + "=" + urls.get(i) + "\n";
					str += "Title" + String.valueOf(i + 1) + "=" + names.get(i) + "\n";
					str += "Length" + String.valueOf(i + 1) + "=" + "-1\n";
				}
				str += "Version=2\n";
				bw.write(str);
				bw.close();
			} else if(file.endsWith(".m3u") || file.endsWith(".txt")) {
                fileOutputStream = new FileOutputStream(filePath);
                outputStreamWriter = new OutputStreamWriter(fileOutputStream, "UTF-8");
                BufferedWriter bw = new BufferedWriter(outputStreamWriter);
				String str = "#EXTM3U\n";
				for(int i = 0; i < names.size(); i++) {
					str += "#EXTINF:0," + names.get(i) + "\n";
					str += urls.get(i) + "\n";
				}
				bw.write(str);
				bw.close();
			} else {
				filePath = filePath + ".m3u";
                fileOutputStream = new FileOutputStream(filePath);
                outputStreamWriter = new OutputStreamWriter(fileOutputStream, "UTF-8");
                BufferedWriter bw = new BufferedWriter(outputStreamWriter);
				String str = "#EXTM3U\n";
				for(int i = 0; i < names.size(); i++) {
					str += "#EXTINF:0," + names.get(i) + "\n";
					str += urls.get(i) + "\n";
				}
				bw.write(str);
				bw.close();
			}
			errorMessage = "";
			return true;
		} catch (IOException e) {
			
			errorMessage = e.getMessage();
			return false;
		}
	};

	//-----------------------------------------------------------------------------
	
};
