package ru.littlevictor.radio;

//-----------------------------------------------------------------------------
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.preference.Preference;
import android.preference.Preference.OnPreferenceClickListener;
import android.preference.PreferenceFragment;
import android.preference.PreferenceGroup;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBarActivity;
import android.support.v7.widget.Toolbar;
import android.text.Editable;
import android.text.InputType;
import android.text.TextWatcher;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.inputmethod.InputMethodManager;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EdgeEffect;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.lang.reflect.Field;

//-----------------------------------------------------------------------------
public class RadioSettings extends ActionBarActivity {
    private static final int DIALOG_NONE = 0;
    private static final int DIALOG_VLC_AUDIO_OUTPUT = 1;
    private static final int DIALOG_VLC_BUFFERING_TIME = 2;
    private static final int DIALOG_AUDIO_PLAYER = 3;
    private static final int DIALOG_APP_THEME = 4;
    private static final int DIALOG_NOTIFY_THEME = 5;
    private static final int DIALOG_WIDGET_THEME = 6;
    private static int dialogIsShow = DIALOG_NONE;
    private static int[] theme;
    private static int themeAccentColor;
    private static int screenMinPixels;
    private static InputMethodManager inputManager;
    private Toolbar actionBar;
    private ActionBarFragment actionBarFragment;
    private FragmentTransaction transaction;

    // -----------------------------------------------------------------------------
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        if(savedInstanceState != null) {
            dialogIsShow = savedInstanceState.getInt("dialogIsShow", DIALOG_NONE);
        }
        screenMinPixels = getScreenMinPixels();
        theme = ThemeChooser.getAppTheme(this);
        themeAccentColor = getResources().getColor(theme[1]);
        inputManager = (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);
        setTheme(theme[2]);
        super.onCreate(null);
        setContentView(R.layout.radio_settings);
        if(theme[0] == R.color.color_theme_primary_1) {
            findViewById(R.id.settings_content).setBackgroundResource(R.color.color_background_list_dark);
        } else {
            findViewById(R.id.settings_content).setBackgroundResource(R.color.color_background_list_light);
        }
        createActionBar(getResources().getConfiguration().orientation);
    };

    //-----------------------------------------------------------------------------
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        actionBar = actionBarFragment.getActionBar();
        actionBar.setNavigationIcon(R.drawable.abc_ic_ab_back_mtrl_am_alpha);
        getFragmentManager().beginTransaction().replace(R.id.settings_content, new SettingsFragment()).commit();
        return true;
    };

    // -----------------------------------------------------------------------------
    private void createActionBar(int orientation) {
        actionBarFragment = new ActionBarFragment();
        actionBarFragment.setParentActivity(this);
        Bundle args = new Bundle();
        if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_LANDSCAPE);
        } else {
            args.putInt(actionBarFragment.LAYOUT_ORIENTATION, actionBarFragment.LAYOUT_PORTRAIT);
        }
        args.putString(actionBarFragment.TITLE, getResources().getString(R.string.app_name));
        args.putString(actionBarFragment.SUBTITLE, getResources().getString(R.string.action_settings));
        actionBarFragment.setArguments(args);
        transaction = getSupportFragmentManager().beginTransaction();
        transaction.replace(R.id.action_bar_container, actionBarFragment);
        transaction.commit();
    };

    // -----------------------------------------------------------------------------
    private int getScreenMinPixels() {
        DisplayMetrics displaymetrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
        int height = displaymetrics.heightPixels;
        int width = displaymetrics.widthPixels;
        if(height <= width) {
            return height;
        } else {
            return width;
        }
    };

    // -----------------------------------------------------------------------------
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                Intent intent = new Intent(this, RadioMain.class).setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
                startActivity(intent);
                finish();
                return true;
        };
        return super.onOptionsItemSelected(item);
    };

    // -----------------------------------------------------------------------------
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_BACK:
                Intent intent = new Intent(this, RadioMain.class).setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK);
                startActivity(intent);
                finish();
                return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    // -----------------------------------------------------------------------------
    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putInt("dialogIsShow", dialogIsShow);
    };

    // -----------------------------------------------------------------------------
    public static class SettingsFragment extends PreferenceFragment {
        private Preference defaultAudioPlayer;
        private Preference vlcAudioOutput;
        private Preference vlcBufferingTime;
        private Preference vlcShowBitrate;
        private Preference appTheme;
        private Preference notifyTheme;
        private Preference widgetTheme;
        private String vlcAudioOutputValue;
        private String[] vlcAudioOutputItems;
        private String vlcBufferingTimeValue;
        private String defaultAudioPlayerValue;
        private String[] defaultAudioPlayerItems;
        private String appThemeValue;
        private String notifyThemeValue;
        private String[] notifyThemeItems;
        private String widgetThemeValue;
        private AlertDialog mDialog;
        private LayoutParams layoutParams;
        private PreferenceGroup vlcPreferenceGroup;

        // -----------------------------------------------------------------------------
        public SettingsFragment() {
            super();
        };

        // -----------------------------------------------------------------------------
        @Override
        public void onActivityCreated(Bundle savedInstanceState) {
            super.onActivityCreated(null);
            ListView list = (ListView) getActivity().findViewById(android.R.id.list);

            if (Build.VERSION.SDK_INT >= 21)
            {
                EdgeEffect edgeEffectTop = new EdgeEffect(getActivity());
                if(theme[0] == R.color.color_theme_primary_1) {
                    edgeEffectTop.setColor(0xffffffff);
                } else {
                    edgeEffectTop.setColor(themeAccentColor);
                }

                EdgeEffect edgeEffectBottom = new EdgeEffect(getActivity());
                if(theme[0] == R.color.color_theme_primary_1) {
                    edgeEffectBottom.setColor(0xffffffff);
                } else {
                    edgeEffectBottom.setColor(themeAccentColor);
                }

                try {
                    Field f1 = AbsListView.class.getDeclaredField("mEdgeGlowTop");
                    f1.setAccessible(true);
                    f1.set(list, edgeEffectTop);

                    Field f2 = AbsListView.class.getDeclaredField("mEdgeGlowBottom");
                    f2.setAccessible(true);
                    f2.set(list, edgeEffectBottom);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                int glowDrawableId = getResources().getIdentifier("overscroll_glow", "drawable", "android");
                Drawable androidGlow = getResources().getDrawable(glowDrawableId);
                if(theme[0] == R.color.color_theme_primary_1) {
                    androidGlow.setColorFilter(0xffffffff, PorterDuff.Mode.SRC_IN);
                } else {
                    androidGlow.setColorFilter(themeAccentColor, PorterDuff.Mode.SRC_IN);
                }
                int edgeDrawableId = getResources().getIdentifier("overscroll_edge", "drawable", "android");
                Drawable androidEdge = getResources().getDrawable(edgeDrawableId);
                if(theme[0] == R.color.color_theme_primary_1) {
                    androidEdge.setColorFilter(0xffffffff, PorterDuff.Mode.SRC_IN);
                } else {
                    androidEdge.setColorFilter(themeAccentColor, PorterDuff.Mode.SRC_IN);
                }
            }
            switch(dialogIsShow) {
                case DIALOG_VLC_AUDIO_OUTPUT:
                    showVlcAudioOutputDialog();
                    break;
                case DIALOG_VLC_BUFFERING_TIME:
                    showVlcBufferingTimeDialog();
                    break;
                case DIALOG_AUDIO_PLAYER:
                    showDefaultAudioPlayerDialog();
                    break;
                case DIALOG_APP_THEME:
                    showAppThemeDialog();
                    break;
                case DIALOG_NOTIFY_THEME:
                    showNotifyThemeDialog();
                    break;
                case DIALOG_WIDGET_THEME:
                    showWidgetThemeDialog();
                    break;
            }
        };

        @Override
        public void onCreate(Bundle paramBundle) {
            super.onCreate(paramBundle);
            addPreferencesFromResource(R.xml.settings);

            vlcPreferenceGroup = (PreferenceGroup) findPreference("preference_category_audio");
            //-----------------------------------------------------------------------------
            vlcAudioOutput = findPreference("vlc_audio_output");
            vlcAudioOutputValue = vlcAudioOutput.getSharedPreferences().getString("vlc_audio_output", "0");
            vlcAudioOutputItems = getResources().getStringArray(R.array.vlc_audio_output_array);
            if(vlcAudioOutputValue.equalsIgnoreCase("1")) {
                vlcAudioOutput.setSummary(vlcAudioOutputItems[1]);
            } else if(vlcAudioOutputValue.equalsIgnoreCase("2")) {
                vlcAudioOutput.setSummary(vlcAudioOutputItems[2]);
            } else {
                vlcAudioOutput.setSummary(vlcAudioOutputItems[0]);
            }
            vlcAudioOutput.setOnPreferenceClickListener(new OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    showVlcAudioOutputDialog();
                    return true;
                }
            });

            //-----------------------------------------------------------------------------
            vlcBufferingTime = findPreference("vlc_buffering_time");
            vlcBufferingTimeValue = vlcBufferingTime.getSharedPreferences().getString("vlc_buffering_time", "0");
            if(vlcBufferingTimeValue.equalsIgnoreCase("0")) {
                vlcBufferingTime.setSummary(getResources().getString(R.string.ticker_auto));
            } else {
                vlcBufferingTime.setSummary(vlcBufferingTimeValue);
            }
            vlcBufferingTime.setOnPreferenceClickListener(new OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    showVlcBufferingTimeDialog();
                    return true;
                }
            });

            //-----------------------------------------------------------------------------
            vlcShowBitrate = findPreference("show_bitrate");

            //-----------------------------------------------------------------------------
            defaultAudioPlayer = findPreference("default_audio_player");
            defaultAudioPlayerValue = defaultAudioPlayer.getSharedPreferences().getString("default_audio_player", "0");
            defaultAudioPlayerItems = getResources().getStringArray(R.array.default_audio_player_array);
            if(defaultAudioPlayerValue.equalsIgnoreCase("1")) {
                defaultAudioPlayer.setSummary(defaultAudioPlayerItems[1]);
                vlcPreferenceGroup.removePreference(vlcAudioOutput);
                vlcPreferenceGroup.removePreference(vlcBufferingTime);
                vlcPreferenceGroup.removePreference(vlcShowBitrate);
            } else if(defaultAudioPlayerValue.equalsIgnoreCase("2")) {
                defaultAudioPlayer.setSummary(defaultAudioPlayerItems[2]);
                vlcPreferenceGroup.removePreference(vlcAudioOutput);
                vlcPreferenceGroup.removePreference(vlcBufferingTime);
                vlcPreferenceGroup.removePreference(vlcShowBitrate);
            } else {
                defaultAudioPlayer.setSummary(defaultAudioPlayerItems[0]);
            }
            defaultAudioPlayer.setOnPreferenceClickListener(new OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    showDefaultAudioPlayerDialog();
                    return true;
                }
            });

            //-----------------------------------------------------------------------------
            appTheme = findPreference("app_theme");
            appThemeValue = appTheme.getSharedPreferences().getString("app_theme", "0");
            appTheme.setOnPreferenceClickListener(new OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    showAppThemeDialog();
                    return true;
                }
            });

            //-----------------------------------------------------------------------------
            notifyTheme = findPreference("notify_theme");
            notifyThemeValue = notifyTheme.getSharedPreferences().getString("notify_theme", "0");
            notifyThemeItems = getResources().getStringArray(R.array.notify_theme_array);
            if(notifyThemeValue.equalsIgnoreCase("1")) {
                notifyTheme.setSummary(notifyThemeItems[1]);
            } else if(notifyThemeValue.equalsIgnoreCase("2")) {
                notifyTheme.setSummary(notifyThemeItems[2]);
            } else if(notifyThemeValue.equalsIgnoreCase("3")) {
                notifyTheme.setSummary(notifyThemeItems[3]);
            } else if(notifyThemeValue.equalsIgnoreCase("4")) {
                notifyTheme.setSummary(notifyThemeItems[4]);
            } else  {
                notifyTheme.setSummary(notifyThemeItems[0]);
            }
            notifyTheme.setOnPreferenceClickListener(new OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    showNotifyThemeDialog();
                    return true;
                }
            });

            //-----------------------------------------------------------------------------
            widgetTheme = findPreference("widget_theme");
            widgetThemeValue = widgetTheme.getSharedPreferences().getString("widget_theme", "0");
            if(widgetThemeValue.equalsIgnoreCase("1")) {
                widgetTheme.setSummary(getResources().getString(R.string.dark_theme));
            } else  {
                widgetTheme.setSummary(getResources().getString(R.string.light_theme));
            }
            widgetTheme.setOnPreferenceClickListener(new OnPreferenceClickListener() {
                @Override
                public boolean onPreferenceClick(Preference preference) {
                    showWidgetThemeDialog();
                    return true;
                }
            });
        };

        // -----------------------------------------------------------------------------
        private void radioServiceReplay() {
            if(RadioService.getState() == RadioService.PLAY) {
                getActivity().sendBroadcast(new Intent(getResources().getString(R.string.media_play_stop)));
                getActivity().sendBroadcast(new Intent(getResources().getString(R.string.media_play_stop)));
            }
        };

        // -----------------------------------------------------------------------------
        private void showVlcAudioOutputDialog() {
            int selectedPosition;
            try {
                selectedPosition = Integer.parseInt(vlcAudioOutputValue);
            } catch(NumberFormatException nfe) {
                selectedPosition = 0;
            }
            final String[] values = getResources().getStringArray(R.array.vlc_audio_output_array_values);
            final LinearLayout dialog = (LinearLayout) getActivity().getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(getActivity()), false);
            dialog.setBackgroundResource(theme[3]);
            final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
            dialogTitle.setText(getText(R.string.vlc_audio_output));
            final ListView list = (ListView) dialog.findViewById(android.R.id.list);
            if(theme[0] == R.color.color_theme_primary_1) {
                list.setBackgroundResource(R.color.color_background_list_dark);
            } else {
                list.setBackgroundResource(R.color.color_background_list_light);
            }
            list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
            list.setAdapter(new AdapterDialogChooseValue(getActivity(), vlcAudioOutputItems, selectedPosition));
            list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                    if(!values[position].equals(vlcAudioOutputValue)) {
                        vlcAudioOutputValue = values[position];
                        vlcAudioOutput.getSharedPreferences().edit().putString("vlc_audio_output", vlcAudioOutputValue).apply();
                        vlcAudioOutput.setSummary(vlcAudioOutputItems[position]);
                        radioServiceReplay();
                    }
                }
            });
            final Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
            btnCancel.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                };
            });
            final AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), theme[4]);
            builder.setView(dialog).setCancelable(true);
            mDialog = builder.create();
            mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
                @Override
                public void onCancel(DialogInterface dialog) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog = null;
                };
            });
            mDialog.show();
            dialogIsShow = DIALOG_VLC_AUDIO_OUTPUT;
        };

        // -----------------------------------------------------------------------------
        private void showVlcBufferingTimeDialog() {
            final LinearLayout dialog = (LinearLayout) getActivity().getLayoutInflater().inflate(R.layout.dialog_prompt, new LinearLayout(getActivity()), false);
            dialog.setBackgroundResource(theme[3]);
            LinearLayout dialogContent = (LinearLayout) dialog.findViewById(R.id.dialog_content);
            if(theme[0] == R.color.color_theme_primary_1) {
                dialogContent.setBackgroundResource(R.color.color_background_list_dark);
            } else {
                dialogContent.setBackgroundResource(R.color.color_background_list_light);
            }
            final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
            dialogTitle.setText(getText(R.string.vlc_buffering_time));
            final TextView dialogDescription = (TextView) dialog.findViewById(R.id.dialog_description);
            dialogDescription.setText(getText(R.string.ticker_vlc_buffering_time));
            final EditText dialogName = (EditText) dialog.findViewById(R.id.dialog_name);
            dialogName.setInputType(InputType.TYPE_CLASS_NUMBER);
            dialogName.setText(vlcBufferingTimeValue);
            final Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
            final Button btnOk = (Button) dialog.findViewById(R.id.btn_ok);
            btnOk.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                    inputManager.hideSoftInputFromWindow((dialog).getWindowToken(), 0);
                    String newValue = dialogName.getText().toString();
                    if(!newValue.equals(vlcBufferingTimeValue)) {
                        int value;
                        try {
                            value = Integer.parseInt(dialogName.getText().toString());
                        } catch(NumberFormatException nfe) {
                            nfe.printStackTrace();
                            value = -1;
                        }
                        if(value >= 0 && value <= 60000) {
                            vlcBufferingTimeValue = new Integer(value).toString();
                            if(vlcBufferingTimeValue.equalsIgnoreCase("0")) {
                                vlcBufferingTime.setSummary(getResources().getString(R.string.ticker_auto));
                            } else {
                                vlcBufferingTime.setSummary(vlcBufferingTimeValue);
                            }
                            vlcBufferingTime.getSharedPreferences().edit().putString("vlc_buffering_time", vlcBufferingTimeValue).apply();
                            radioServiceReplay();
                        } else {
                            vlcBufferingTime.getSharedPreferences().edit().putString("vlc_buffering_time", "0").apply();
                            showToast(getResources().getString(R.string.ticker_vlc_buffering_time), Toast.LENGTH_SHORT);
                        }
                    }
                };
            });
            btnCancel.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                    inputManager.hideSoftInputFromWindow((dialog).getWindowToken(), 0);
                };
            });
            final AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), theme[4]);
            builder.setView(dialog).setCancelable(true);
            mDialog = builder.create();
            mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
                @Override
                public void onCancel(DialogInterface dialog) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog = null;
                };
            });
            mDialog.show();
            dialogName.addTextChangedListener(new TextWatcher() {
                public void afterTextChanged(Editable s) {
                    if (s.length() > 0) {
                        btnOk.setEnabled(true);
                    } else {
                        btnOk.setEnabled(false);
                    }
                }
                public void beforeTextChanged(CharSequence s, int start, int count, int after) {
                }
                public void onTextChanged(CharSequence s, int start, int before, int count) {
                }
            });
            dialogIsShow = DIALOG_VLC_BUFFERING_TIME;
        };

        // -----------------------------------------------------------------------------
        private void showDefaultAudioPlayerDialog() {
            int selectedPosition;
            try {
                selectedPosition = Integer.parseInt(defaultAudioPlayerValue);
            } catch(NumberFormatException nfe) {
                selectedPosition = 0;
            }
            final String[] values = getResources().getStringArray(R.array.default_audio_player_array_values);
            final LinearLayout dialog = (LinearLayout) getActivity().getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(getActivity()), false);
            dialog.setBackgroundResource(theme[3]);
            final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
            dialogTitle.setText(getText(R.string.default_audio_player));
            final ListView list = (ListView) dialog.findViewById(android.R.id.list);
            if(theme[0] == R.color.color_theme_primary_1) {
                list.setBackgroundResource(R.color.color_background_list_dark);
            } else {
                list.setBackgroundResource(R.color.color_background_list_light);
            }
            list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
            list.setAdapter(new AdapterDialogChooseValue(getActivity(), defaultAudioPlayerItems, selectedPosition));
            list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                    if(!defaultAudioPlayerValue.equals(values[position])) {
                        defaultAudioPlayerValue = values[position];
                        defaultAudioPlayer.getSharedPreferences().edit().putString("default_audio_player", defaultAudioPlayerValue).apply();
                        defaultAudioPlayer.setSummary(defaultAudioPlayerItems[position]);
                        if(defaultAudioPlayerValue.equalsIgnoreCase("1")) {
                            vlcPreferenceGroup.removePreference(vlcAudioOutput);
                            vlcPreferenceGroup.removePreference(vlcBufferingTime);
                            vlcPreferenceGroup.removePreference(vlcShowBitrate);
                            radioServiceReplay();
                        } else if(defaultAudioPlayerValue.equalsIgnoreCase("2")) {
                            vlcPreferenceGroup.removePreference(vlcAudioOutput);
                            vlcPreferenceGroup.removePreference(vlcBufferingTime);
                            vlcPreferenceGroup.removePreference(vlcShowBitrate);
                        } else {
                            vlcPreferenceGroup.addPreference(vlcAudioOutput);
                            vlcPreferenceGroup.addPreference(vlcBufferingTime);
                            vlcPreferenceGroup.addPreference(vlcShowBitrate);
                            radioServiceReplay();
                        }
                    }
                }
            });
            final Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
            btnCancel.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                };
            });
            final AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), theme[4]);
            builder.setView(dialog).setCancelable(true);
            mDialog = builder.create();
            mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
                @Override
                public void onCancel(DialogInterface dialog) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog = null;
                };
            });
            mDialog.show();
            dialogIsShow = DIALOG_AUDIO_PLAYER;
        };

        // -----------------------------------------------------------------------------
        private void showAppThemeDialog() {
            int selectedPosition;
            try {
                selectedPosition = Integer.parseInt(appThemeValue);
            } catch(NumberFormatException nfe) {
                selectedPosition = 0;
            }
            final String[] items = getResources().getStringArray(R.array.app_theme_array);

            final LinearLayout dialog = (LinearLayout) getActivity().getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(getActivity()), false);
            dialog.setBackgroundResource(theme[3]);

            final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
            dialogTitle.setText(getText(R.string.app_theme));
            final ListView list = (ListView) dialog.findViewById(android.R.id.list);
            if(theme[0] == R.color.color_theme_primary_1) {
                list.setBackgroundResource(R.color.color_background_list_dark);
            } else {
                list.setBackgroundResource(R.color.color_background_list_light);
            }
            list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
            list.setAdapter(new AdapterDialogChooseColor(getActivity(), items, selectedPosition));
            layoutParams = list.getLayoutParams();
            float listItemHeight = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, getResources().getInteger(R.integer.preference_color_item_height), getResources().getDisplayMetrics());
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                if(listItemHeight*list.getCount() > screenMinPixels/100*60) {
                    layoutParams.height = (int)(screenMinPixels/100*60);
                    list.setLayoutParams(layoutParams);
                }
            }
            list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    if(!appThemeValue.equalsIgnoreCase(items[position])) {
                        appThemeValue = items[position];
                        appTheme.getSharedPreferences().edit().putString("app_theme", appThemeValue).apply();
                        dialogIsShow = DIALOG_NONE;
                        mDialog.dismiss();
                        mDialog = null;
                        getActivity().startService(new Intent(getActivity(), RadioService.class).putExtra(getResources().getString(R.string.service_state), RadioService.STATE));
                        getActivity().finish();
                        startActivity(getActivity().getIntent());
                        return;
                    }
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                }
            });
            final Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
            btnCancel.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                };
            });
            final AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), theme[4]);
            builder.setView(dialog).setCancelable(true);
            mDialog = builder.create();
            mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
                @Override
                public void onCancel(DialogInterface dialog) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog = null;
                };
            });
            mDialog.show();
            dialogIsShow = DIALOG_APP_THEME;
        };

        // -----------------------------------------------------------------------------
        private void showNotifyThemeDialog() {
            int selectedPosition;
            try {
                selectedPosition = Integer.parseInt(notifyThemeValue);
            } catch(NumberFormatException nfe) {
                selectedPosition = 0;
            }
            final String[] values = getResources().getStringArray(R.array.notify_theme_array_values);
            final LinearLayout dialog = (LinearLayout) getActivity().getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(getActivity()), false);
            dialog.setBackgroundResource(theme[3]);
            final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
            dialogTitle.setText(getText(R.string.notify_theme));
            final ListView list = (ListView) dialog.findViewById(android.R.id.list);
            if(theme[0] == R.color.color_theme_primary_1) {
                list.setBackgroundResource(R.color.color_background_list_dark);
            } else {
                list.setBackgroundResource(R.color.color_background_list_light);
            }
            list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
            list.setAdapter(new AdapterDialogChooseValue(getActivity(), notifyThemeItems, selectedPosition));
            layoutParams = list.getLayoutParams();
            float listItemHeight = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, getResources().getInteger(R.integer.preference_color_item_height), getResources().getDisplayMetrics());
            if (getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
                if(listItemHeight*list.getCount() > screenMinPixels/100*60) {
                    layoutParams.height = (int)(screenMinPixels/100*60);
                    list.setLayoutParams(layoutParams);
                }
            }
            list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                    if(!notifyThemeValue.equalsIgnoreCase(values[position])) {
                        notifyThemeValue = values[position];
                        notifyTheme.getSharedPreferences().edit().putString("notify_theme", notifyThemeValue).apply();
                        notifyTheme.setSummary(notifyThemeItems[position]);
                        getActivity().startService(new Intent(getActivity(), RadioService.class).putExtra(getResources().getString(R.string.service_state), RadioService.STATE));
                    }
                }
            });
            final Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
            btnCancel.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                };
            });
            final AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), theme[4]);
            builder.setView(dialog).setCancelable(true);
            mDialog = builder.create();
            mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
                @Override
                public void onCancel(DialogInterface dialog) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog = null;
                };
            });
            mDialog.show();
            dialogIsShow = DIALOG_NOTIFY_THEME;
        };

        // -----------------------------------------------------------------------------
        private void showWidgetThemeDialog() {
            int selectedPosition;
            try {
                selectedPosition = Integer.parseInt(widgetThemeValue);
            } catch(NumberFormatException nfe) {
                selectedPosition = 0;
            }
            final String[] items = getResources().getStringArray(R.array.widget_theme_array);
            final String[] values = getResources().getStringArray(R.array.widget_theme_array_values);
            final LinearLayout dialog = (LinearLayout) getActivity().getLayoutInflater().inflate(R.layout.dialog_choose, new LinearLayout(getActivity()), false);
            dialog.setBackgroundResource(theme[3]);
            final TextView dialogTitle = (TextView) dialog.findViewById(R.id.dialog_title);
            dialogTitle.setText(getText(R.string.widget_theme));
            final ListView list = (ListView) dialog.findViewById(android.R.id.list);
            if(theme[0] == R.color.color_theme_primary_1) {
                list.setBackgroundResource(R.color.color_background_list_dark);
            } else {
                list.setBackgroundResource(R.color.color_background_list_light);
            }
            list.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
            list.setAdapter(new AdapterDialogChooseValue(getActivity(), items, selectedPosition));
            list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                    if(!widgetThemeValue.equalsIgnoreCase(values[position])) {
                        widgetThemeValue = values[position];
                        widgetTheme.getSharedPreferences().edit().putString("widget_theme", widgetThemeValue).apply();
                        widgetTheme.setSummary(items[position]);
                        Intent intent = new Intent(getActivity(), RadioService.class)
                                .putExtra(getResources().getString(R.string.service_state), RadioService.STATE);
                        getActivity().startService(intent);
                    }
                }
            });
            final Button btnCancel = (Button) dialog.findViewById(R.id.btn_cancel);
            btnCancel.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog.dismiss();
                    mDialog = null;
                };
            });
            final AlertDialog.Builder builder = new AlertDialog.Builder(getActivity(), theme[4]);
            builder.setView(dialog).setCancelable(true);
            mDialog = builder.create();
            mDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {
                @Override
                public void onCancel(DialogInterface dialog) {
                    dialogIsShow = DIALOG_NONE;
                    mDialog = null;
                };
            });
            mDialog.show();
            dialogIsShow = DIALOG_WIDGET_THEME;
        };

        // -----------------------------------------------------------------------------
        private void showToast(String message, int length) {
            Toast toast = Toast.makeText(getActivity(), message, length);
            toast.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL, 0, 0);
            toast.show();
        };

        // -----------------------------------------------------------------------------
        @Override
        public void onDestroy() {
            if(mDialog != null) {
                mDialog.dismiss();
            }
            dialogIsShow = DIALOG_NONE;
            super.onDestroy();
        };
    };
};
